"use strict";
const command_1 = require("@oclif/command");
const file_system_transformer_1 = require("./core/file-system-transformer");
const manager_1 = require("./core/manager");
const svg_parser_1 = require("./core/svg-parser");
const json_output_handler_1 = require("./core/json-output-handler");
const messenger_1 = require("./utils/messenger");
const config_factory_1 = require("./utils/config-factory");
const s_icon_error_1 = require("./errors/s-icon-error");
const cli_logger_1 = require("./utils/cli-logger");
const colors = require("./utils/colors");
class SvgToJson extends command_1.Command {
    constructor(argv, config) {
        super(argv, config);
        this.defaults = {
            source: '',
            destination: process.cwd(),
            allowedExtensions: ['.svg'],
        };
        this.messenger = new messenger_1.Messenger();
        this.configFactory = new config_factory_1.ConfigFactory({
            messenger: this.messenger,
            defaultConfigPath: `${process.cwd()}/icons.config.js`,
        });
        this.cliLogger = new cli_logger_1.CliLogger();
    }
    async run() {
        this.cliLogger.header();
        const { flags } = this.parse(SvgToJson);
        this.listenMessenger();
        const config = Object.assign(Object.assign(Object.assign({}, this.defaults), this.configFactory.resolveConfig(await this.configFactory.readConfigFile(flags.config))), this.configFactory.resolveConfig(flags));
        if (!config.source) {
            config.source = await this.cliLogger.prompt(colors.info('`source`') +
                ' was not declared, please fill your icons source directory');
        }
        const manager = new manager_1.Manager({
            transformer: new file_system_transformer_1.FileSystemTransformer({
                allowedExtensions: config.allowedExtensions,
                sourcePath: config.source,
                messenger: this.messenger,
            }),
            parser: new svg_parser_1.SvgParser({ messenger: this.messenger }),
            outputHandler: new json_output_handler_1.JsonOutputHandler({
                destinationPath: config.destination,
                messenger: this.messenger,
            }),
        });
        try {
            manager.transform().parse().output();
        }
        catch (error) {
            if (!(error instanceof s_icon_error_1.SIconError)) {
                throw error;
            }
            this.messenger.error(error.message);
        }
        this.cliLogger.footer();
    }
    listenMessenger() {
        this.messenger.listen({
            [messenger_1.Messenger.LOG]: (message) => this.cliLogger.log(message ? message : ''),
            [messenger_1.Messenger.ERROR]: (message) => this.cliLogger.die(message ? message : ''),
            [messenger_1.Messenger.WARN]: (message) => this.cliLogger.error(message ? message : ''),
        });
    }
}
SvgToJson.description = 'describe the command here';
SvgToJson.flags = {
    version: command_1.flags.version({ char: 'v' }),
    help: command_1.flags.help({ char: 'h' }),
    force: command_1.flags.boolean({ char: 'f' }),
    source: command_1.flags.string({
        char: 's',
        description: 'source path to svg icons directory.',
    }),
    allowedExtensions: command_1.flags.string({
        char: 'a',
        description: 'allowed extensions.',
    }),
    destination: command_1.flags.string({
        char: 'd',
        description: 'output destination path.',
    }),
    config: command_1.flags.string({
        char: 'c',
        description: 'config file.',
    }),
};
module.exports = SvgToJson;
